
{----------------------------------------------------------------------------------------------}
{                                                                                              }
{ Ce programme montre l'utilisation des unites LookupAccount, lookupLuid, lookupPrivileges et  }
{ TokenInfos.                                                                                  }
{                                                                                              }
{ Il affiche le nom du compte utilisateur courant, les groupes auxquels il appartient, ainsi   }
{ que ses privileges et leur etat. Puis, il tente d'activer le privilege SE_BACKUP_NAME, pour  }
{ sauvegarder la SAM sur le disque dur.                                                        }
{                                                                                              }
{ Pour toutes remarques --> phaz at dune2 point info                                           }
{                                                                                              }
{    ~ Phaz ~                                                                                  }
{                                                                                              }
{----------------------------------------------------------------------------------------------}



PROGRAM Demo;

{$APPTYPE CONSOLE}


Uses
  SysUtils,                         // Necessaire pour utiliser Format et SysErrorMessage
  windows,                          // Necessaire pour HKey
  LookupAccount in 'LookupAccount.pas',
  lookupLuid in 'LookupLuid.pas',
  lookupPrivileges in 'LookupPrivileges.pas',
  TokenInfos in 'TokenInfos.pas',
  ANSI_ASCII in 'ANSI_ASCII.pas' ;  // Pour afficher correctement les caracteres accentues
                                    // dans une fenetre console.


Procedure DipslayError (Error : Cardinal) ;
  Begin
    Writeln (ANSI2ASCII ('Erreur n'#176' '), error, ANSI2ASCII (' : '+SysErrorMessage (Error)));
    Writeln ;
  End ;

Procedure Display (Msg : String) ;
  Begin
    Write (ANSI2ASCII (Msg)) ;
  End ;

Procedure DisplayLn (Msg : String) ;
  Begin
    Writeln (ANSI2ASCII (Msg)) ;
  End ;


Var
  Domain  : String ;
  Name    : String ;
  FCible  : String ;
  SIDtype : Cardinal ;
  Error   : Cardinal ;
  Lang    : Cardinal ;
  n       : Cardinal ;
  User    : PSid ;
  owner   : PSid ;
  infos   : TPrivilegeInfos ;
  Group   : TGroupInfo ;
  Admin   : Boolean ;
  HSubKey : HKey ;



Begin
  Admin := False ;
  
  Writeln ;
  Displayln ('===============================================================================');
  Displayln('========================= Informations sur l''utilisateur ======================');
  Displayln ('===============================================================================');
  Writeln ;

  // Recuperation des informations sur l'utilisateur dans le Token
  If GetTokenUser (GetCurrentProcess, User, Error)
    Then Begin
      If ((GetAccountName ('', User, Domain, SIDType, Error)) <> 'NUL' ) AND (Error = 0)
        Then Begin
          Displayln ('Le Token de ce processus appartient a : ');
          Writeln ;
          Displayln (' Compte            : ' + GetAccountName ('',User,Domain,SIDType,Error));
          Displayln (' SID               : ' + SID2STR (User)) ;
          Displayln (' Membre du domaine : ' + Domain) ;
          Displayln (' Type              : ' + SidType2STR (SIDType)) ;
        End Else
          Displayln ('La recuperation du SID de l''utilisateur s''est deroulee normalement,'+
                     ' mais une erreur s''est produite durant la recherche du nom chaine du'+
                     ' compte')
    End Else DipslayError (Error) ;
  If User <> NIL Then HeapFree (GetProcessHeap, 0, User) ;


  Writeln ;
  Displayln ('===============================================================================');
  Writeln ;

  // Recuperation des infos sur le proprietaire par defaut des nouveaux objets crees.
  If GetTokenOwner (GetCurrentProcess, Owner, Error)
    Then Begin
      If ( (GetAccountName ('', Owner, Domain, SIDType, Error)) <> 'NUL' ) AND (Error = 0)
        Then Begin
          Displayln ('Le proprietaire des nouveaux objets crees par cet utilisateur est : ');
          Writeln ;
          Displayln (' Compte            : ' + GetAccountName ('',Owner,Domain,SIDType,Error));
          Displayln (' SID               : ' + SID2STR (Owner));
          Displayln (' Membre du domaine : ' + Domain);
          Displayln (' Type              : ' + SidType2STR (SIDType));
        End Else
          Displayln ('La recuperation du SID du proprietaire par defaut de tous les nouveaux '+
                     'objets crees s''est deroulee normalement, mais une erreur s''est '+
                     'produite durant la recherche du nom chaine du compte') ;
    End Else DipslayError (Error) ;
  If Owner  <> NIL Then HeapFree (GetProcessHeap, 0, Owner) ;

  Writeln ;
  Display ('Pressez ''Entree'' pour continuer') ;
  Readln ;

  Writeln ;
  Displayln ('===============================================================================');
  Displayln ('=========================== Liste des groupes du Token ========================');
  Displayln ('===============================================================================');
  Writeln ;


  Displayln ('Cet utilisateur est membre des groupes suivants : ') ;
  Writeln ; Writeln ;

  N := 0;
  Repeat
    Group.Sid := NIL ;
    EnumTokenGroups (GetCurrentProcess, n, Group, Error) ;
    If (Error = 0) Then
      Begin
        Display (Format ('%-50.50S', [SID2Str (Group.Sid)])) ;
        Displayln (': ' + Format ('%-25.25s', [Group.Name])) ;
        Display (' --> ') ;
        Display (Format ('%-25.25S', [SidType2Str (Group.SidType)])) ;
        If Group.Enabled then Display (' Enabled ') ;
        If Group.ByDefault Then Display (' by default ');
        If Group.Mandatory Then Display (' Mandatory ') ;
        If Group.LogonId Then Display (' Logon Id ') ;
        If Group.Owner Then Display (' Owner ') ;
        If Group.Resource Then Display (' Ressource ') ;
        If Group.UseForDenyOnly Then Display (' Use for denny only ') ;

        // Cette ligne permet de tester si l'utilisateur est un administrateur du systeme local
        // (mais ne detectera pas l'administrateur d'un controleur de domaine)
        If Copy (Sid2Str (Group.Sid), length (Sid2Str (Group.Sid)) - 2 , 3) = '544'
          Then Admin := True ;

        Writeln ; Writeln ;
      End ;
    n := n + 1 ;
    If Group.Sid <> NIL Then HeapFree (GetProcessHeap, 0, Group.Sid) ;
  Until (Error <> 0) ;

  If Error <> ERROR_NO_MORE_ITEMS Then DipslayError (Error) ;

  If Admin Then
    Begin
      Writeln ;
      Displayln ('L''utilisateur est un administrateur du systeme local') ;
    End Else
    Begin
      Writeln ;
      Displayln ('L''utilisateur n''est pas administrateur du systeme local') ;
    End ;

  Readln  ;

  Writeln ;
  Displayln ('===============================================================================');
  Displayln ('========================= Liste des privileges du Token =======================');
  Displayln ('===============================================================================');
  Writeln ;

  Displayln ('Cet utilisateur possede les privileges suivants : ') ;
  Writeln ; Writeln ;

  n := 0 ;
  EnumTokenPrivileges (GetCurrentProcess, n, Infos, Error) ;
  While Error = 0 Do
    Begin
      Display ('LUID : ' + (format ('%-8d', [Int64 (Infos.iLUID)])) +
               '  NAME : ' + (format ('%-32.32s', [Infos.Name]))) ;
      If Infos.Enabled   Then Display (' -> Enabled') Else Display (' -> Disabled') ;
      If Infos.Bydefault Then Display (' by Default');
      If Infos.Used      Then Display (' already used') ;
      If Infos.Removed   Then Display (' remove') ;
      Writeln ;
      N := N + 1 ;
      EnumTokenPrivileges (GetCurrentProcess, n, Infos, Error) ;
    End ;

  If Error <> ERROR_NO_MORE_ITEMS Then DipslayError (Error) ;

  Writeln ;
  Displayln ('===============================================================================');
  Displayln ('=========================== Sauvegarde de la SAM ==============================');
  Displayln ('===============================================================================');
  Writeln ;

  // Recuperation du nom de SE_BACKUP_NAME dans la langue de l'utilisateur
  Name := Privilege2UserStr ('',SE_BACKUP_NAME, Lang, Error) ;
  If Error = 0
    Then Displayln ('Dans la langue de l''utilisateur SE_BACKUP_NAME s''appelle : '
                   + #13#10 + '"' + Name + '"')
    Else DipslayError (Error) ;

  // Test de l'etat du privilege
  If (PrivilegeEnabled (GetCurrentProcess, SE_BACKUP_NAME, error)) AND (Error = 0)
    Then Displayln ('  --> Actuellement ce privilege est active')
    Else Displayln ('  --> Actuellement ce privilege est desactive') ;

  If Error <> 0 Then
    Begin
      Displayln ('L''erreur suivante s''est produite pendant le test du privilege') ;
      DipslayError (Error) ;
    End ;

  Writeln ;
  Display ('Appuyez sur ''entree'' pour tenter de l''activer') ;
  Readln ;

  // Activation de SE_BACKUP_NAME pour etre autorise a sauvegarder une clef du registre
  If SetPrivilegeAttribute (GetCurrentProcess, SE_BACKUP_NAME, TRUE, Error) Then
    Begin
      Displayln ('  --> Ce privilege a ete active avec succes') ;
      Writeln ;
    End Else
    Begin
      Displayln ('  --> L''activation de ce privilege a echoue') ;
      DipslayError (Error) ;
    End ;

  FCible := 'C:\KEY.DAT' ;
  Displayln ('Appuyez sur ''entree'' pour tenter d''enregister la SAM dans :') ;
  Display (FCible) ;
  Readln ;

  // Sauvegarde de "HKEY_LOCAL_MACHINE/SAM" dans FCible
  Error := RegOpenKeyEx (HKEY_LOCAL_MACHINE, PChar ('SAM'), 0,  KEY_QUERY_VALUE , HSubKey) ;
  If Error = ERROR_SUCCESS Then
    Error := RegSaveKey (HSubKey, PCHar (FCible), NIL) ;
  If HSubKey <> 0 Then RegCloseKey (HSubKey) ;
  If Error = ERROR_SUCCESS
    Then Begin
      Displayln ('  --> Sauvegarde de la SAM effectuee avec succes') ;
      Writeln ;
    End
    Else Begin
      Displayln ('  --> La sauvegarde de la SAM a echoue :');
      DipslayError (Error) ;
    End ;

  Display ('Appuyez sur ''entree'' pour desactiver le privilege' + #10#13 + '"' + Name + '"');
  Readln  ;

  // Desactivation du privilege apres utilisation
  If SetPrivilegeAttribute (GetCurrentProcess, SE_BACKUP_NAME, FALSE, Error )
    Then Displayln ('  --> Ce privilege a ete desactive avec succes')
    Else Begin
      Displayln ('  --> La desactivation de ce privilege a echoue') ;
      DipslayError (Error) ;
      Writeln ;
    End ;

  Writeln ;
  Displayln ('===============================================================================');
  Writeln ;

  writeln ;
  Displayln ('Pressez ''Entree'' pour terminer') ;
  readln ;
End.






